@@ -180,6 +180,9 @@ ok(tmp === "undefined", "typeof((new Object).doesnotexist = " + tmp);
 tmp = typeof(testObj.onlyDispID);
 ok(tmp === "unknown", "typeof(testObj.onlyDispID) = " + tmp);
 
+ok("\0\0x\0\0".length === 5, "\"\\0\\0x\\0\\0\".length = " + "\0\0x\0\0".length);
+ok("\0\0x\0\0" === String.fromCharCode(0) + "\0x\0" + String.fromCharCode(0), "\"\\0\\0x\\0\\0\" unexpected");
+
 ok(testFunc1(true, "test") === true, "testFunc1 not returned true");
 
 ok(testFunc1.arguments === null, "testFunc1.arguments = " + testFunc1.arguments);
@@ -1183,6 +1186,37 @@ case 3:
     expect(ret, "try");
 })();
 
+(function() {
+    var e;
+    var E_FAIL = -2147467259;
+    var JS_E_SUBSCRIPT_OUT_OF_RANGE = -2146828279;
+
+    try {
+        throwInt(E_FAIL);
+    }catch(ex) {
+        e = ex;
+    }
+    ok(e.name === "Error", "e.name = " + e.name);
+    ok(e.message === "", "e.message = " + e.message);
+    ok(e.number === E_FAIL, "e.number = " + e.number);
+
+    try {
+        throwInt(JS_E_SUBSCRIPT_OUT_OF_RANGE);
+    }catch(ex) {
+        e = ex;
+    }
+    ok(e.name === "RangeError", "e.name = " + e.name);
+    ok(e.number === JS_E_SUBSCRIPT_OUT_OF_RANGE, "e.number = " + e.number);
+
+    try {
+        throwEI(JS_E_SUBSCRIPT_OUT_OF_RANGE);
+    }catch(ex) {
+        e = ex;
+    }
+    ok(e.name === "RangeError", "e.name = " + e.name);
+    ok(e.number === JS_E_SUBSCRIPT_OUT_OF_RANGE, "e.number = " + e.number);
+})();
+
 tmp = eval("1");
 ok(tmp === 1, "eval(\"1\") !== 1");
 eval("{ ok(tmp === 1, 'eval: tmp !== 1'); } tmp = 2;");
@@ -1523,7 +1557,7 @@ deleteTest = 1;
 delete deleteTest;
 try {
     tmp = deleteTest;
-    ok(false, "deleteTest not throwed exception?");
+    ok(false, "deleteTest did not throw an exception?");
 }catch(ex) {}
 
 (function() {
