@@ -126,6 +126,42 @@ static const struct object_ops thread_apc_ops =
 };
 
 
+/* thread CPU context */
+
+struct context
+{
+    struct object   obj;        /* object header */
+    unsigned int    status;     /* status of the context */
+    context_t       regs;       /* context data */
+};
+
+static void dump_context( struct object *obj, int verbose );
+static int context_signaled( struct object *obj, struct wait_queue_entry *entry );
+
+static const struct object_ops context_ops =
+{
+    sizeof(struct context),     /* size */
+    dump_context,               /* dump */
+    no_get_type,                /* get_type */
+    add_queue,                  /* add_queue */
+    remove_queue,               /* remove_queue */
+    context_signaled,           /* signaled */
+    no_satisfied,               /* satisfied */
+    no_signal,                  /* signal */
+    no_get_fd,                  /* get_fd */
+    no_map_access,              /* map_access */
+    default_get_sd,             /* get_sd */
+    default_set_sd,             /* set_sd */
+    no_lookup_name,             /* lookup_name */
+    no_link_name,               /* link_name */
+    NULL,                       /* unlink_name */
+    no_open_file,               /* open_file */
+    no_kernel_obj_list,         /* get_kernel_obj_list */
+    no_close_handle,            /* close_handle */
+    no_destroy                  /* destroy */
+};
+
+
 /* thread operations */
 
 static void dump_thread( struct object *obj, int verbose );
@@ -180,7 +216,6 @@ static inline void init_thread_structure( struct thread *thread )
     thread->unix_pid        = -1;  /* not known yet */
     thread->unix_tid        = -1;  /* not known yet */
     thread->context         = NULL;
-    thread->suspend_context = NULL;
     thread->teb             = 0;
     thread->entry_point     = 0;
     thread->debug_ctx       = NULL;
@@ -222,6 +257,35 @@ static inline int is_valid_address( client_ptr_t addr )
     return addr && !(addr % sizeof(int));
 }
 
+
+/* dump a context on stdout for debugging purposes */
+static void dump_context( struct object *obj, int verbose )
+{
+    struct context *context = (struct context *)obj;
+    assert( obj->ops == &context_ops );
+
+    fprintf( stderr, "context flags=%x\n", context->regs.flags );
+}
+
+
+static int context_signaled( struct object *obj, struct wait_queue_entry *entry )
+{
+    struct context *context = (struct context *)obj;
+    return context->status != STATUS_PENDING;
+}
+
+
+static struct context *create_thread_context( struct thread *thread )
+{
+    struct context *context;
+    if (!(context = alloc_object( &context_ops ))) return NULL;
+    context->status = STATUS_PENDING;
+    memset( &context->regs, 0, sizeof(context->regs) );
+    context->regs.cpu = thread->process->cpu;
+    return context;
+}
+
+
 /* create a new thread */
 struct thread *create_thread( int fd, struct process *process, const struct security_descriptor *sd )
 {
@@ -318,6 +382,13 @@ static void cleanup_thread( struct thread *thread )
 {
     int i;
 
+    if (thread->context)
+    {
+        thread->context->status = STATUS_ACCESS_DENIED;
+        wake_up( &thread->context->obj, 0 );
+        release_object( thread->context );
+        thread->context = NULL;
+    }
     clear_apc_queue( &thread->system_apc );
     clear_apc_queue( &thread->user_apc );
     free( thread->req_data );
@@ -325,7 +396,6 @@ static void cleanup_thread( struct thread *thread )
     if (thread->request_fd) release_object( thread->request_fd );
     if (thread->reply_fd) release_object( thread->reply_fd );
     if (thread->wait_fd) release_object( thread->wait_fd );
-    free( thread->suspend_context );
     cleanup_clipboard_thread(thread);
     destroy_thread_windows( thread );
     free_msg_queue( thread );
@@ -344,8 +414,6 @@ static void cleanup_thread( struct thread *thread )
     thread->request_fd = NULL;
     thread->reply_fd = NULL;
     thread->wait_fd = NULL;
-    thread->context = NULL;
-    thread->suspend_context = NULL;
     thread->desktop = 0;
     thread->desc = NULL;
     thread->desc_len = 0;
@@ -583,17 +651,12 @@ static void set_thread_info( struct thread *thread,
 /* stop a thread (at the Unix level) */
 void stop_thread( struct thread *thread )
 {
-    if (thread->context) return;  /* already inside a debug event, no need for a signal */
+    if (thread->context) return;  /* already suspended, no need for a signal */
+    if (!(thread->context = create_thread_context( thread ))) return;
     /* can't stop a thread while initialisation is in progress */
     if (is_process_init_done(thread->process)) send_thread_signal( thread, SIGUSR1 );
 }
 
-/* stop a thread if it's supposed to be suspended */
-void stop_thread_if_suspended( struct thread *thread )
-{
-    if (thread->suspend + thread->process->suspend > 0) stop_thread( thread );
-}
-
 /* suspend a thread */
 int suspend_thread( struct thread *thread )
 {
@@ -780,6 +843,18 @@ static int send_thread_wakeup( struct thread *thread, client_ptr_t cookie, int s
     struct wake_up_reply reply;
     int ret;
 
+    /* check if we're waking current suspend wait */
+    if (thread->context && thread->suspend_cookie == cookie
+        && signaled != STATUS_KERNEL_APC && signaled != STATUS_USER_APC)
+    {
+        if (!thread->context->regs.flags)
+        {
+            release_object( thread->context );
+            thread->context = NULL;
+        }
+        else signaled = STATUS_KERNEL_APC; /* signal a fake APC so that client calls select to get a new context */
+    }
+
     memset( &reply, 0, sizeof(reply) );
     reply.cookie   = cookie;
     reply.signaled = signaled;
@@ -1198,7 +1273,6 @@ void kill_thread( struct thread *thread, int violent_death )
 static void copy_context( context_t *to, const context_t *from, unsigned int flags )
 {
     assert( to->cpu == from->cpu );
-    to->flags |= flags;
     if (flags & SERVER_CTX_CONTROL) to->ctl = from->ctl;
     if (flags & SERVER_CTX_INTEGER) to->integer = from->integer;
     if (flags & SERVER_CTX_SEGMENTS) to->seg = from->seg;
@@ -1402,7 +1476,7 @@ DECL_HANDLER(init_thread)
     reply->version = SERVER_PROTOCOL_VERSION;
     reply->server_start = server_start_time;
     reply->all_cpus     = supported_cpus & get_prefix_cpu_mask();
-    reply->suspend      = (current->suspend || process->suspend);
+    reply->suspend      = (current->suspend || process->suspend || current->context != NULL);
     return;
 
  error:
@@ -1536,18 +1610,38 @@ DECL_HANDLER(select)
     struct thread_apc *apc;
     const apc_result_t *result = get_req_data();
 
-    if (get_req_data_size() < sizeof(*result))
+    if (get_req_data_size() < sizeof(*result) ||
+        get_req_data_size() - sizeof(*result) < req->size ||
+        req->size & 3)
     {
         set_error( STATUS_INVALID_PARAMETER );
         return;
     }
+
+    if (get_req_data_size() - sizeof(*result) - req->size == sizeof(context_t))
+    {
+        const context_t *context = (const context_t *)((const char *)(result + 1) + req->size);
+        if ((current->context && current->context->status != STATUS_PENDING) || context->cpu != current->process->cpu)
+        {
+            set_error( STATUS_INVALID_PARAMETER );
+            return;
+        }
+
+        if (!current->context && !(current->context = create_thread_context( current ))) return;
+        copy_context( &current->context->regs, context,
+                      context->flags & ~(current->context->regs.flags | get_context_system_regs(current->process->cpu)) );
+        current->context->status = STATUS_SUCCESS;
+        current->suspend_cookie = req->cookie;
+        wake_up( &current->context->obj, 0 );
+    }
+
     if (!req->cookie)
     {
         set_error( STATUS_INVALID_PARAMETER );
         return;
     }
 
-    op_size = min( get_req_data_size() - sizeof(*result), sizeof(select_op) );
+    op_size = min( req->size, sizeof(select_op) );
     memset( &select_op, 0, sizeof(select_op) );
     memcpy( &select_op, result + 1, op_size );
 
@@ -1609,6 +1703,13 @@ DECL_HANDLER(select)
         }
         release_object( apc );
     }
+    else if (get_error() != STATUS_PENDING && get_reply_max_size() == sizeof(context_t) &&
+             current->context && current->suspend_cookie == req->cookie)
+    {
+        if (current->context->regs.flags) set_reply_data( &current->context->regs, sizeof(context_t) );
+        release_object( current->context );
+        current->context = NULL;
+    }
 }
 
 /* queue an APC for a thread or process */
@@ -1712,6 +1813,8 @@ DECL_HANDLER(get_apc_result)
 /* retrieve the current context of a thread */
 DECL_HANDLER(get_thread_context)
 {
+    struct context *thread_context = NULL;
+    unsigned int system_flags;
     struct thread *thread;
     context_t *context;
 
@@ -1720,35 +1823,58 @@ DECL_HANDLER(get_thread_context)
         set_error( STATUS_INVALID_PARAMETER );
         return;
     }
-    if (!(thread = get_thread_from_handle( req->handle, THREAD_GET_CONTEXT ))) return;
-    reply->self = (thread == current);
 
-    if (thread != current && !thread->context)
+    if ((thread_context = (struct context *)get_handle_obj( current->process, req->handle, 0, &context_ops )))
     {
-        /* thread is not suspended, retry (if it's still running) */
+        close_handle( current->process, req->handle ); /* avoid extra server call */
+        system_flags = get_context_system_regs( thread_context->regs.cpu );
+    }
+    else if ((thread = get_thread_from_handle( req->handle, THREAD_GET_CONTEXT )))
+    {
+        clear_error();
+        system_flags = get_context_system_regs( thread->process->cpu );
         if (thread->state == RUNNING)
         {
-            set_error( STATUS_PENDING );
-            if (req->suspend)
+            reply->self = (thread == current);
+            if (thread != current) stop_thread( thread );
+            if (thread->context)
             {
-                release_object( thread );
-                /* make sure we have suspend access */
-                if (!(thread = get_thread_from_handle( req->handle, THREAD_SUSPEND_RESUME ))) return;
-                suspend_thread( thread );
+                /* make sure that system regs are valid in thread context */
+                if (req->flags & system_flags & ~thread->context->regs.flags)
+                    get_thread_context( thread, &thread->context->regs, req->flags & system_flags );
+                if (!get_error()) thread_context = (struct context *)grab_object( thread->context );
+            }
+            else if (!get_error() && (context = set_reply_data_size( sizeof(context_t) )))
+            {
+                assert( reply->self );
+                memset( context, 0, sizeof(context_t) );
+                context->cpu = thread_context->regs.cpu;
+                if (req->flags & system_flags)
+                {
+                    get_thread_context( thread, context, req->flags & system_flags );
+                    context->flags |= req->flags & system_flags;
+                }
             }
         }
         else set_error( STATUS_UNSUCCESSFUL );
+        release_object( thread );
     }
-    else if ((context = set_reply_data_size( sizeof(context_t) )))
-    {
-        unsigned int flags = get_context_system_regs( thread->process->cpu );
+    if (get_error() || !thread_context) return;
 
+    set_error( thread_context->status );
+    if (!thread_context->status && (context = set_reply_data_size( sizeof(context_t) )))
+    {
         memset( context, 0, sizeof(context_t) );
-        context->cpu = thread->process->cpu;
-        if (thread->context) copy_context( context, thread->context, req->flags & ~flags );
-        if (req->flags & flags) get_thread_context( thread, context, req->flags & flags );
+        context->cpu = thread_context->regs.cpu;
+        copy_context( context, &thread_context->regs, req->flags );
+        context->flags = req->flags;
     }
-    release_object( thread );
+    else if (thread_context->status == STATUS_PENDING)
+    {
+        reply->handle = alloc_handle( current->process, thread_context, SYNCHRONIZE, 0 );
+    }
+
+    release_object( thread_context );
 }
 
 /* set the current context of a thread */
@@ -1765,84 +1891,25 @@ DECL_HANDLER(set_thread_context)
     if (!(thread = get_thread_from_handle( req->handle, THREAD_SET_CONTEXT ))) return;
     reply->self = (thread == current);
 
-    if (thread != current && !thread->context)
-    {
-        /* thread is not suspended, retry (if it's still running) */
-        if (thread->state == RUNNING)
-        {
-            set_error( STATUS_PENDING );
-            if (req->suspend)
-            {
-                release_object( thread );
-                /* make sure we have suspend access */
-                if (!(thread = get_thread_from_handle( req->handle, THREAD_SUSPEND_RESUME ))) return;
-                suspend_thread( thread );
-            }
-        }
-        else set_error( STATUS_UNSUCCESSFUL );
-    }
-    else if (context->cpu == thread->process->cpu)
+    if (thread->state == TERMINATED) set_error( STATUS_UNSUCCESSFUL );
+    else if (context->cpu != thread->process->cpu) set_error( STATUS_INVALID_PARAMETER );
+    else
     {
         unsigned int system_flags = get_context_system_regs(context->cpu) & context->flags;
         unsigned int client_flags = context->flags & ~system_flags;
 
         if (system_flags) set_thread_context( thread, context, system_flags );
-        if (thread->context && !get_error()) copy_context( thread->context, context, client_flags );
+        if (thread != current && !get_error()) stop_thread( thread );
+        if (thread->context && !get_error())
+        {
+            copy_context( &thread->context->regs, context, context->flags );
+            thread->context->regs.flags |= client_flags;
+        }
     }
-    else set_error( STATUS_INVALID_PARAMETER );
 
     release_object( thread );
 }
 
-/* retrieve the suspended context of a thread */
-DECL_HANDLER(get_suspend_context)
-{
-    if (get_reply_max_size() < sizeof(context_t))
-    {
-        set_error( STATUS_INVALID_PARAMETER );
-        return;
-    }
-
-    if (current->suspend_context)
-    {
-        if (current->suspend_context->flags)
-            set_reply_data_ptr( current->suspend_context, sizeof(context_t) );
-        else
-            free( current->suspend_context );
-        if (current->context == current->suspend_context)
-        {
-            current->context = NULL;
-            stop_thread_if_suspended( current );
-        }
-        current->suspend_context = NULL;
-    }
-    else set_error( STATUS_INVALID_PARAMETER );  /* not suspended, shouldn't happen */
-}
-
-/* store the suspended context of a thread */
-DECL_HANDLER(set_suspend_context)
-{
-    const context_t *context = get_req_data();
-
-    if (get_req_data_size() < sizeof(context_t))
-    {
-        set_error( STATUS_INVALID_PARAMETER );
-        return;
-    }
-
-    if (current->context || context->cpu != current->process->cpu)
-    {
-        /* nested suspend or exception, shouldn't happen */
-        set_error( STATUS_INVALID_PARAMETER );
-    }
-    else if ((current->suspend_context = mem_alloc( sizeof(context_t) )))
-    {
-        memcpy( current->suspend_context, get_req_data(), sizeof(context_t) );
-        current->suspend_context->flags = 0;  /* to keep track of what is modified */
-        current->context = current->suspend_context;
-    }
-}
-
 /* fetch a selector entry for a thread */
 DECL_HANDLER(get_selector_entry)
 {
